searchData={"items":[{"type":"module","title":"FermatPrimalityTest","doc":"A probabilistic primality test implementation based on Fermat's Little Theorem.","ref":"FermatPrimalityTest.html"},{"type":"module","title":"Overview - FermatPrimalityTest","doc":"This module implements the Fermat primality test, a probabilistic algorithm for determining\nwhether a given number is likely to be prime. The test is based on Fermat's Little Theorem,\nwhich states that if $p$ is a prime number and $a$ is any integer not divisible by $p$,\nthen:\n\n$$a^{p-1} \\equiv 1 \\pmod{p}$$","ref":"FermatPrimalityTest.html#module-overview"},{"type":"module","title":"Algorithm - FermatPrimalityTest","doc":"For a given number $n$:\n1. If $n = 1$, return `false` (1 is not prime)\n2. If $n = 2$, return `true` (2 is prime)\n3. For numbers $n > 2$:\n   - Generate 100 random bases $a$ where $1 < a < n$\n   - For each base, check if $\\gcd(a, n) = 1$ (coprime)\n   - If coprime, compute $a^{n-1} \\bmod n$\n   - If the result is not 1, $n$ is definitely composite\n   - If all 100 tests pass, $n$ is probably prime","ref":"FermatPrimalityTest.html#module-algorithm"},{"type":"module","title":"Mathematical Foundation - FermatPrimalityTest","doc":"The test relies on the contrapositive of Fermat's Little Theorem: if there exists an $a$\ncoprime to $n$ such that $a^{n-1} \\not\\equiv 1 \\pmod{n}$, then $n$ is composite.\n\nHowever, the converse is not always true. There exist composite numbers called\n**Fermat pseudoprimes** that pass the test for some bases. Carmichael numbers are\ncomposite numbers that pass the test for all bases coprime to them.","ref":"FermatPrimalityTest.html#module-mathematical-foundation"},{"type":"module","title":"Accuracy - FermatPrimalityTest","doc":"- **True positives**: All prime numbers will pass the test\n- **False positives**: Some composite numbers (pseudoprimes) may pass the test\n- **False negatives**: None (if the test says composite, it's definitely composite)\n\nThe probability of a false positive decreases with the number of bases tested.\nWith 100 random bases, the probability of a false positive is extremely low for\nmost practical purposes.","ref":"FermatPrimalityTest.html#module-accuracy"},{"type":"module","title":"Performance - FermatPrimalityTest","doc":"The algorithm uses efficient modular exponentiation with $O(\\log n)$ complexity\nfor each base test. The overall complexity is $O(k \\log n)$ where $k$ is the\nnumber of bases tested (100 in this implementation).","ref":"FermatPrimalityTest.html#module-performance"},{"type":"module","title":"Examples - FermatPrimalityTest","doc":"iex> FermatPrimalityTest.of(2)\n    true\n\n    iex> FermatPrimalityTest.of(3)\n    true\n\n    iex> FermatPrimalityTest.of(4)\n    false\n\n    iex> FermatPrimalityTest.of(17)\n    true\n\n    iex> FermatPrimalityTest.of(100)\n    false","ref":"FermatPrimalityTest.html#module-examples"},{"type":"module","title":"Limitations - FermatPrimalityTest","doc":"- This is a probabilistic test, not deterministic\n- Very large numbers may have higher false positive rates\n- For cryptographic applications, consider using deterministic tests like AKS\n- The test may be slow for very large numbers due to modular exponentiation","ref":"FermatPrimalityTest.html#module-limitations"},{"type":"module","title":"References - FermatPrimalityTest","doc":"- Fermat's Little Theorem\n- Modular Exponentiation\n- Probabilistic Primality Testing","ref":"FermatPrimalityTest.html#module-references"},{"type":"function","title":"FermatPrimalityTest.mod_pow/3","doc":"Calculates modular exponentiation using the square-and-multiply algorithm.\n\nThis function efficiently computes $base^{exp} \\bmod mod$ using the binary\nexponentiation method, also known as the square-and-multiply algorithm.","ref":"FermatPrimalityTest.html#mod_pow/3"},{"type":"function","title":"Mathematical Background - FermatPrimalityTest.mod_pow/3","doc":"The algorithm works by expressing the exponent in binary and using the identity:\n$a^{2k} = (a^k)^2$ and $a^{2k+1} = a \\cdot (a^k)^2$\n\nThis reduces the number of multiplications from $O(exp)$ to $O(\\log exp)$.","ref":"FermatPrimalityTest.html#mod_pow/3-mathematical-background"},{"type":"function","title":"Parameters - FermatPrimalityTest.mod_pow/3","doc":"- `base` - The base number\n- `exp` - The exponent (must be non-negative)\n- `mod` - The modulus","ref":"FermatPrimalityTest.html#mod_pow/3-parameters"},{"type":"function","title":"Returns - FermatPrimalityTest.mod_pow/3","doc":"The result of $base^{exp} \\bmod mod$","ref":"FermatPrimalityTest.html#mod_pow/3-returns"},{"type":"function","title":"Examples - FermatPrimalityTest.mod_pow/3","doc":"iex> FermatPrimalityTest.mod_pow(10, 2, 3)\n    1\n\n    iex> FermatPrimalityTest.mod_pow(7, 3, 5)\n    3\n\n    iex> FermatPrimalityTest.mod_pow(2, 10, 1000)\n    24\n\n    iex> FermatPrimalityTest.mod_pow(5, 0, 7)\n    1","ref":"FermatPrimalityTest.html#mod_pow/3-examples"},{"type":"function","title":"Algorithm - FermatPrimalityTest.mod_pow/3","doc":"1. If $exp = 0$, return 1\n2. If $exp < 0$, raise ArithmeticError\n3. Use square-and-multiply algorithm:\n   - Initialize result = 1\n   - For each bit in the binary representation of exp:\n     - Square the current result: $result = result^2 \\bmod mod$\n     - If the bit is 1, multiply by base: $result = result \\cdot base \\bmod mod$","ref":"FermatPrimalityTest.html#mod_pow/3-algorithm"},{"type":"function","title":"Time Complexity - FermatPrimalityTest.mod_pow/3","doc":"$O(\\log exp)$ - logarithmic in the exponent.","ref":"FermatPrimalityTest.html#mod_pow/3-time-complexity"},{"type":"function","title":"Space Complexity - FermatPrimalityTest.mod_pow/3","doc":"$O(1)$ - constant space usage.","ref":"FermatPrimalityTest.html#mod_pow/3-space-complexity"},{"type":"function","title":"Raises - FermatPrimalityTest.mod_pow/3","doc":"- `ArithmeticError` when the exponent is negative","ref":"FermatPrimalityTest.html#mod_pow/3-raises"},{"type":"function","title":"FermatPrimalityTest.of/1","doc":"Tests whether a number is prime using the Fermat primality test.\n\nThis function implements a probabilistic primality test based on Fermat's Little Theorem.\nIt tests the number against 100 random bases to determine primality.","ref":"FermatPrimalityTest.html#of/1"},{"type":"function","title":"Parameters - FermatPrimalityTest.of/1","doc":"- `n` - The integer to test for primality","ref":"FermatPrimalityTest.html#of/1-parameters"},{"type":"function","title":"Returns - FermatPrimalityTest.of/1","doc":"- `true` if the number is probably prime\n- `false` if the number is definitely composite","ref":"FermatPrimalityTest.html#of/1-returns"},{"type":"function","title":"Examples - FermatPrimalityTest.of/1","doc":"iex> FermatPrimalityTest.of(1)\n    false\n\n    iex> FermatPrimalityTest.of(2)\n    true\n\n    iex> FermatPrimalityTest.of(3)\n    true\n\n    iex> FermatPrimalityTest.of(4)\n    false\n\n    iex> FermatPrimalityTest.of(17)\n    true\n\n    iex> FermatPrimalityTest.of(100)\n    false","ref":"FermatPrimalityTest.html#of/1-examples"},{"type":"function","title":"Algorithm Details - FermatPrimalityTest.of/1","doc":"1. **Edge cases**: 1 returns `false`, 2 returns `true`\n2. **Random base generation**: Creates 100 random bases $a$ where $1 < a < n$\n3. **Coprimality check**: Ensures $\\gcd(a, n) = 1$ using Lehmer's GCD algorithm\n4. **Fermat test**: Computes $a^{n-1} \\bmod n$ using efficient modular exponentiation\n5. **Decision**: If any test fails, returns `false`; if all pass, returns `true`","ref":"FermatPrimalityTest.html#of/1-algorithm-details"},{"type":"function","title":"Time Complexity - FermatPrimalityTest.of/1","doc":"$O(k \\log n)$ where $k = 100$ (number of bases) and $n$ is the input number.","ref":"FermatPrimalityTest.html#of/1-time-complexity"},{"type":"function","title":"Space Complexity - FermatPrimalityTest.of/1","doc":"$O(1)$ - constant space usage regardless of input size.","ref":"FermatPrimalityTest.html#of/1-space-complexity"},{"type":"extras","title":"Fermat Primality Test","doc":"# Fermat Primality Test\n\nA probabilistic primality test implementation in Elixir based on Fermat's Little Theorem.","ref":"readme.html"},{"type":"extras","title":"Overview - Fermat Primality Test","doc":"This library provides an efficient implementation of the Fermat primality test, a probabilistic algorithm for determining whether a given number is likely to be prime. The test is based on Fermat's Little Theorem and uses modular exponentiation for optimal performance.","ref":"readme.html#overview"},{"type":"extras","title":"Mathematical Foundation - Fermat Primality Test","doc":"Fermat's Little Theorem states that if $p$ is a prime number and $a$ is any integer not divisible by $p$, then:\n\n$$a^{p-1} \\equiv 1 \\pmod{p}$$\n\nThe test works by checking this congruence for multiple random bases. If any base fails the test, the number is definitely composite. If all bases pass, the number is probably prime.","ref":"readme.html#mathematical-foundation"},{"type":"extras","title":"Features - Fermat Primality Test","doc":"- **Probabilistic Testing**: Uses 100 random bases for high accuracy\n- **Efficient Implementation**: $O(k \\log n)$ time complexity where $k = 100$\n- **Mathematical Rigor**: Based on well-established number theory\n- **Comprehensive Documentation**: Full API documentation with examples\n- **Quality Assurance**: Includes static analysis, type checking, and spell checking","ref":"readme.html#features"},{"type":"extras","title":"Installation - Fermat Primality Test","doc":"Add `fermat_primality_test` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:fermat_primality_test, \"~> 0.1\"}\n  ]\nend\n```\n\nThen install dependencies:\n\n```bash\nmix deps.get\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Fermat Primality Test","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Basic Usage - Fermat Primality Test","doc":"```elixir\n# Test if a number is prime\nFermatPrimalityTest.of(17)  # Returns true\nFermatPrimalityTest.of(100) # Returns false\nFermatPrimalityTest.of(1)   # Returns false\nFermatPrimalityTest.of(2)   # Returns true\n```","ref":"readme.html#basic-usage"},{"type":"extras","title":"Modular Exponentiation - Fermat Primality Test","doc":"```elixir\n# Calculate modular exponentiation\nFermatPrimalityTest.mod_pow(10, 2, 3)  # Returns 1\nFermatPrimalityTest.mod_pow(7, 3, 5)   # Returns 3\nFermatPrimalityTest.mod_pow(2, 10, 1000) # Returns 24\n```","ref":"readme.html#modular-exponentiation"},{"type":"extras","title":"Algorithm Details - Fermat Primality Test","doc":"","ref":"readme.html#algorithm-details"},{"type":"extras","title":"Fermat Primality Test - Fermat Primality Test","doc":"For a given number $n$:\n\n1. **Edge Cases**: \n   - If $n = 1$, return `false` (1 is not prime)\n   - If $n = 2$, return `true` (2 is prime)\n\n2. **Main Test** (for $n > 2$):\n   - Generate 100 random bases $a$ where $1 < a < n$\n   - For each base, check if $\\gcd(a, n) = 1$ (coprime)\n   - If coprime, compute $a^{n-1} \\bmod n$\n   - If the result is not 1, $n$ is definitely composite\n   - If all 100 tests pass, $n$ is probably prime","ref":"readme.html#fermat-primality-test"},{"type":"extras","title":"Modular Exponentiation - Fermat Primality Test","doc":"The library uses the square-and-multiply algorithm for efficient modular exponentiation:\n\n- **Time Complexity**: $O(\\log exp)$\n- **Space Complexity**: $O(1)$\n- **Algorithm**: Binary exponentiation with modular arithmetic","ref":"readme.html#modular-exponentiation-1"},{"type":"extras","title":"Accuracy - Fermat Primality Test","doc":"- **True Positives**: All prime numbers will pass the test\n- **False Positives**: Some composite numbers (pseudoprimes) may pass the test\n- **False Negatives**: None (if the test says composite, it's definitely composite)\n\nThe probability of a false positive decreases with the number of bases tested. With 100 random bases, the probability is extremely low for most practical purposes.","ref":"readme.html#accuracy"},{"type":"extras","title":"Performance - Fermat Primality Test","doc":"- **Time Complexity**: $O(k \\log n)$ where $k = 100$ (number of bases)\n- **Space Complexity**: $O(1)$ - constant space usage\n- **Optimizations**: Efficient modular exponentiation and GCD calculations","ref":"readme.html#performance"},{"type":"extras","title":"Limitations - Fermat Primality Test","doc":"- **Probabilistic**: This is not a deterministic test\n- **Pseudoprimes**: Some composite numbers may pass the test\n- **Large Numbers**: Performance may degrade for very large numbers\n- **Cryptographic Use**: For cryptographic applications, consider deterministic tests like AKS","ref":"readme.html#limitations"},{"type":"extras","title":"License - Fermat Primality Test","doc":"Copyright (c) 2025 University of Kitakyushu\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at [http://www.apache.org/licenses/LICENSE-2.0](http://www.apache.org/licenses/LICENSE-2.0)\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"readme.html#license"},{"type":"extras","title":"References - Fermat Primality Test","doc":"- [Fermat's Little Theorem](https://en.wikipedia.org/wiki/Fermat%27s_little_theorem)\n- [Modular Exponentiation](https://en.wikipedia.org/wiki/Modular_exponentiation)\n- [Probabilistic Primality Testing](https://en.wikipedia.org/wiki/Primality_test#Probabilistic_tests)\n- [Carmichael Numbers](https://en.wikipedia.org/wiki/Carmichael_number)","ref":"readme.html#references"},{"type":"extras","title":"Changelog - Fermat Primality Test","doc":"See [CHANGELOG.md](CHANGELOG.md) for a list of changes and version history.","ref":"readme.html#changelog"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright [yyyy] [name of copyright owner]\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [1.0.0] - 2025-07-21","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- **Core Algorithm**: Fermat primality test implementation\n  - Probabilistic testing with 100 random bases\n  - Efficient modular exponentiation using square-and-multiply algorithm\n  - Edge case handling for numbers 1 and 2\n  - Integration with LehmerGcd for coprimality testing\n\n- **Main Functions**:\n  - `FermatPrimalityTest.of/1` - Main primality test function\n  - `FermatPrimalityTest.mod_pow/3` - Modular exponentiation implementation\n\n- **Documentation**:\n  - Comprehensive module documentation with mathematical background\n  - Detailed function documentation with examples\n  - Type specifications (@spec) for better tooling support\n  - Professional README with usage examples and algorithm explanations\n\n- **Development Tools**:\n  - Credo configuration for static analysis\n  - Dialyzer setup for type checking\n  - Spellweaver integration for documentation quality\n  - ExDoc configuration with KaTeX math rendering\n  - Comprehensive test suite\n\n- **Project Structure**:\n  - Apache 2.0 license\n  - Professional README with installation and usage instructions\n  - Development workflow with `mix check` alias\n  - Package metadata for Hex.pm publishing\n  - Initial project structure and setup\n  - Comprehensive documentation with mathematical formulas\n  - Quality assurance tooling (Credo, Dialyzer, Spellweaver)\n  - KaTeX integration for mathematical formula rendering","ref":"changelog.html#added"},{"type":"extras","title":"Technical Details - Changelog","doc":"#### Algorithm Implementation\n- **Fermat Primality Test**: Based on Fermat's Little Theorem\n  - Time Complexity: $O(k \\log n)$ where $k = 100$\n  - Space Complexity: $O(1)$\n  - Accuracy: No false negatives, extremely low false positive rate\n\n- **Modular Exponentiation**: Square-and-multiply algorithm\n  - Time Complexity: $O(\\log exp)$\n  - Space Complexity: $O(1)$\n  - Handles edge cases (zero exponent, negative exponent)\n\n#### Mathematical Foundation\n- Implements Fermat's Little Theorem: $a^{p-1} \\equiv 1 \\pmod{p}$\n- Uses contrapositive for composite number detection\n- Handles Fermat pseudoprimes and Carmichael numbers\n- Probabilistic approach with 100 random bases\n\n#### Quality Assurance\n- **Static Analysis**: Credo with custom rules\n- **Type Checking**: Dialyzer with ignore file\n- **Documentation**: ExDoc with KaTeX math rendering\n- **Spell Checking**: Spellweaver with technical terms\n- **Code Formatting**: Mix format integration","ref":"changelog.html#technical-details"},{"type":"extras","title":"Dependencies - Changelog","doc":"- **Development**: ex_doc, dialyxir, credo, spellweaver, nstandard\n- **Runtime**: nstandard (for LehmerGcd integration)","ref":"changelog.html#dependencies"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"None - This is the initial release.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Deprecations - Changelog","doc":"None - This is the initial release.","ref":"changelog.html#deprecations"},{"type":"extras","title":"Version History - Changelog","doc":"### [1.0.0] - 2025-07-21 - Initial Release\n- Complete Fermat primality test implementation\n- Comprehensive documentation with mathematical formulas\n- Professional development tooling and quality assurance\n- Full project structure with license, README, and metadata\n\n---","ref":"changelog.html#version-history"},{"type":"extras","title":"Release Notes - Changelog","doc":"","ref":"changelog.html#release-notes"},{"type":"extras","title":"Version 1.0.0 - Changelog","doc":"This is the initial stable release of the Fermat Primality Test library. The implementation provides a robust, well-documented probabilistic primality test suitable for educational and research purposes.\n\n**Key Features:**\n- Efficient probabilistic primality testing\n- Comprehensive mathematical documentation\n- Professional development tooling\n- Type-safe implementation with Dialyzer support\n\n**Use Cases:**\n- Educational demonstrations of Fermat's Little Theorem\n- Research and experimentation with primality testing\n- Learning probabilistic algorithms\n- Mathematical library development\n\n**Limitations:**\n- Probabilistic nature (not deterministic)\n- May have false positives for very large numbers\n- Not suitable for cryptographic applications requiring deterministic tests\n\n---","ref":"changelog.html#version-1-0-0"},{"type":"extras","title":"Contributing - Changelog","doc":"When adding new features or making changes, please update this changelog following the [Keep a Changelog](https://keepachangelog.com/en/1.0.0/) format.","ref":"changelog.html#contributing"},{"type":"extras","title":"Changelog Entry Format - Changelog","doc":"```markdown\n## [Version] - YYYY-MM-DD","ref":"changelog.html#changelog-entry-format"},{"type":"extras","title":"Added - Changelog","doc":"- New features","ref":"changelog.html#added-1"},{"type":"extras","title":"Changed - Changelog","doc":"- Changes in existing functionality","ref":"changelog.html#changed"},{"type":"extras","title":"Deprecated - Changelog","doc":"- Soon-to-be removed features","ref":"changelog.html#deprecated"},{"type":"extras","title":"Removed - Changelog","doc":"- Removed features","ref":"changelog.html#removed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Bug fixes","ref":"changelog.html#fixed"},{"type":"extras","title":"Security - Changelog","doc":"- Vulnerability fixes\n```","ref":"changelog.html#security"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}